

#include "pev.h"


#define MAX_ROC_BANKS  16
#define MAX_THITS     256

#define ADC_SLOT	7
#define TDC_SLOT_1	6
#define TDC_SLOT_2	5


/* global variable declarations */
Tcl_Interp* interp;
char TCL_PATH[256]=DEFAULT_TCL_PATH;

primex_t primex;


int continuous;
int USE_ADC=1;
int SHOW_DOTS=0;
int reset_histos=0;
int REQUIRE_TAGGER;
int SUM_ENERGY_FRAC;
int ADCSUM_CUT=-1000;
int NUM_EVENTS_READ=0;
int NUM_PHYSICS_EVENTS=0;

char SESSION[64]="";
char INPUT_FILE[256]="";
FILE *CODA_FILE;
CODA_event_id_t *EVENT_ID;
CODA_header_t *EVENT_HEAD;
TDC1877_t TDC[MAX_THITS];
int N_TDC_HITS;
int ADC[64];

typedef struct {
	TDC1877_t x,y;
}XY_hit_t;

TDC1877_t XHITS[MAX_THITS];
TDC1877_t YHITS[MAX_THITS];
XY_hit_t XYHITS[MAX_THITS];
int N_XHITS,N_YHITS,N_XYHITS;


unsigned int PEDESTAL[64];


/* function declarations */
int ReadConfiguration(int run_number);
void ParseCommandLineArgs(int narg,char *argv[]);
int GetEvent(void);
void Usage(void);


int main(int narg,char* argv[])
{
   char str[256],fname[256],cmd[256],*ptr;
   int i;
   static int one=1,two=2,three=3,four=4;
   int N,err;
   auto char TCL_LIBRARY[256]="TCL_LIBRARY=";
   
   /*-------------- Print version info etc --------*/
   printf("\n----------- PEV (Primex Event Viewer) ----------- \n");
   printf("Version 2.0 (Aug.2001): \ncompiled: ");
   ansi_red();    printf("%s %s on "
      ,__DATE__,__TIME__);  ansi_normal();
   ansi_blue();printf("%s\n",COMPILE_HOST);ansi_normal();
   printf("Tcl/Tk versions %s/%s\n",TCL_VERSION,TK_VERSION);
   for(i=1;i<narg;i++)if(!strcmp(argv[i],"-v"))return 0;
   printf("\n------------------------------------------------- \n\n");
   
   /*--------------- initialize -----------------*/   
   
   /* Tcl/Tk */
   putenv(TCL_LIBRARY); /* unset the TCL_LIBRARY variable */
   interp=Tcl_CreateInterp();
   if(!interp){printf("Unable to create Tcl/Tk interpreter!\n");exit(-1);}
   Tcl_Init(interp);
   Tk_Init(interp);
   
   
   /* define path to tcl code directory */
   if(ptr=getenv("PEV_TCL_PATH"))
      sprintf(TCL_PATH,"%s",ptr);
   else {
      printf("\n No \"PEV_TCL_PATH\" environment variable set. Defaulting to\n");
      printf(" \"%s\".\n\n",TCL_PATH);
   }
   
   /* Read Tcl source and Link C-tcl routines */
   sprintf(fname,"%s/pev.tcl",TCL_PATH);
   if(Tcl_EvalFile(interp,fname)!=TCL_OK)
      printf("Error in \"%s\":\n%s\n",fname,interp->result);

#define TRFLAGS TCL_GLOBAL_ONLY | TCL_TRACE_WRITES
   Tcl_CreateCommand(interp,"DrawEvent",&TCL_DrawEvent,0,NULL);
   err=Tcl_LinkVar(interp,"continuous",(char*)&continuous,TCL_LINK_INT);
   err=Tcl_LinkVar(interp,"use_adc",(char*)&USE_ADC,TCL_LINK_INT);
   err=Tcl_LinkVar(interp,"show_dots",(char*)&SHOW_DOTS,TCL_LINK_INT);
   err=Tcl_LinkVar(interp,"reset_histos",(char*)&reset_histos,TCL_LINK_INT);
   err=Tcl_LinkVar(interp,"sum_energy_frac",(char*)&SUM_ENERGY_FRAC,TCL_LINK_INT);
   err=Tcl_LinkVar(interp,"require_tagger",(char*)&REQUIRE_TAGGER,TCL_LINK_INT);
   err=Tcl_LinkVar(interp,"ADCSUM_CUT",(char*)&ADCSUM_CUT,TCL_LINK_INT);

   
	/*---------- Parse Command line args ---------*/
	ParseCommandLineArgs(narg,argv);

	/*----------- Open event Source (file or ET) --------------*/
	if(strlen(SESSION)){
		if(init_et(SESSION)){
			fprintf(stderr,"unable to connect to ET (session=\"%s\") !\n\n",SESSION);
			return -1;
		}
	}else if(strlen(INPUT_FILE)){
		/* Open the input file. Assume NO header so first byte starts CODA data */
		CODA_FILE=OpenCODAFile(INPUT_FILE);
		if(!CODA_FILE){
			fprintf(stderr,"unable to open file \"%s\" for reading!\n\n",INPUT_FILE);
			return -1;
		}
	}else{
		fprintf(stderr,"No Event source!\n\n");
		Usage();
	}

	/*----------- Read calibration constants --------------*/
	ReadConfiguration(0);

   /*----------------- main loop -----------------*/
   printf("Initializing...\n");
   sprintf(cmd,"Initialize %d %d",ARRAYSIZE,600/ARRAYSIZE);
   Tcl_Eval(interp,cmd);
   printf("%s\n",interp->result);
   printf("\nDrawing grid...\n");
   Tcl_Eval(interp,"DrawGrid");
   Tcl_Eval(interp,"DrawXYGrid");
   printf("%s\n",interp->result);
   printf("starting main loop...\n");
   Tk_MainLoop();
   
   
	/*----------- completion and clean-up ---------*/
	if(strlen(SESSION)){
		end_et();
	}else{
		CloseCODAFile(CODA_FILE);
	}

	printf("pev finished.\n");
	printf(" %d events read total\n\n",NUM_EVENTS_READ);
	return 0;
}


/* this is to satisfy potential linking problems using various f77s */
int MAIN__(int narg,char* argv[]){return main(narg,argv);}
int __main(int narg,char* argv[]){return main(narg,argv);}


int TCL_DrawEvent(	 ClientData clientData
   			,Tcl_Interp *interp
                        ,int argc,char *argv[])
{
   static long Nevents=0L;
   int i,j;
   char cmd[512];
   int x,y,f;
   int insertion;
   char fill[32];
   char outline[32];
   char TAG[]="event";
   int recnum=0;
   float Edep=100000.0,T,E_TAG;
   int err;
   float X=0.0,Y=0.0;
   int Ndots;
   static float last_tac_x=-100,last_tac_y=-100;
   int ADCSUM=0;


   /* read an event */
   err=GetEvent();
   if(err<0){
      printf("\nNo more events.\n");
      Tcl_Eval(interp,"update;update;after idle DrawEvent");
      return TCL_OK;
   }else if(err){
      Tcl_Eval(interp,"update;update;after idle DrawEvent");
      return TCL_OK;
   }
	/* Sum of all channels. Don't include cells reading only pedestals. */   
   for(x=0;x<ARRAYSIZE;x++){
      for(y=0;y<ARRAYSIZE;y++){
      	if(abs(ADC[xy2id(x,y)])>=50)ADCSUM+=ADC[xy2id(x,y)];
      }
   }
   Edep=(float)ADCSUM;
   
   
   /* clear the former event */
   sprintf(cmd,".c.c delete %s",TAG);
   Tcl_Eval(interp,cmd);

   for(x=0;x<ARRAYSIZE;x++){
      for(y=0;y<ARRAYSIZE;y++){

		/* Don't draw cells within 50 ADC counts of pedestal */
		if(ADC[xy2id(x,y)]<50)continue;
         
         f=(int)(256.0*(float)ADC[xy2id(x,y)]/(float)ADCSUM);
         if(f>0xFF)f=0xFF;
         f=0xFF-f;
         if(f>0xD0)f=0xD0;
         sprintf(fill,"#%2.2x%2.2x%2.2x",f,f,0xFF);
         sprintf(outline,"#%x%x%x",0,0,0);
         
         /* color the cell */
         sprintf(cmd,"DrawCell %d %d %s %s {%s rec%d} %d"
               ,x ,y ,fill, outline ,TAG, ++recnum,1);
         Tcl_Eval(interp,cmd);
         
         /* bind a command to display values with mouseovers */
         sprintf(cmd,".c.c bind rec%d <Enter> ",recnum);
 			sprintf(cmd,"%s{.i.energy configure -text \"ADC: %d\"",cmd,ADC[xy2id(x,y)]);
         sprintf(cmd,"%s ; .i.id configure -text \"id: %3d\"}" ,cmd,x+1+(y*ARRAYSIZE));
         Tcl_Eval(interp,cmd);
      }
   }

   /* Find Center of Energy */
   for(x=0;x<ARRAYSIZE;x++){
      for(y=0;y<ARRAYSIZE;y++){
         
		/* Don't include cells within 50 ADC counts of pedestal */
		if(ADC[xy2id(x,y)]<50)continue;

         X+=((float)x+0.5)*(float)ADC[xy2id(x,y)]/Edep;
         Y+=((float)y+0.5)*(float)ADC[xy2id(x,y)]/Edep;

      }
   }

	Nevents++;
	
	/* Display dots */
	if(SHOW_DOTS){
		Ndots=100;
		i=Nevents%Ndots;
		if(Nevents>Ndots){
			sprintf(cmd,".c.c delete dot%d",i);
			Tcl_Eval(interp,cmd);
		}
		sprintf(fill,"#%x%x%x",0,0,0);
		sprintf(cmd,"DrawDot %f %f %s {dot%d dots}",X,Y,fill,i);
		Tcl_Eval(interp,cmd);
		/* printf("%s\n",interp->result); */
		sprintf(cmd,".c.c raise dots %s",TAG);
		Tcl_Eval(interp,cmd);
	}else{
		Tcl_Eval(interp,".c.c delete dots");
	}


	/* Event values */
	if(USE_ADC){
		sprintf(cmd,".c.s.di.edep configure -text \"ADC sum: %d\"",(int)ADCSUM);
	}else{
		sprintf(cmd,".c.s.di.edep configure -text \"Edeposited: %4.3f GeV\"",(float)ADCSUM);   
	}
	Tcl_Eval(interp,cmd);
	sprintf(cmd,".c.s.di.run configure -text \"RUN: %d \"",-1);
	Tcl_Eval(interp,cmd);
	sprintf(cmd,".c.s.di.evnt configure -text \"Event: %d \"",EVENT_ID ? EVENT_ID->event_number:-1);
	Tcl_Eval(interp,cmd);
 
	/* individual detectors */
	for(x=0;x<ARRAYSIZE;x++){
		for(y=0;y<ARRAYSIZE;y++){
			i=x+(y*ARRAYSIZE);
			sprintf(cmd,".c.s.erg.lb delete %d",i);
			Tcl_Eval(interp,cmd);
			sprintf(cmd,".c.s.erg.lb insert %d \"ADC(x=%d,y=%d): %d\"",i,x+1,y+1,ADC[xy2id(x,y)]);
			Tcl_Eval(interp,cmd);
		}
	}

	/* X-Y coordinate detector */
	Tcl_Eval(interp,".c.s.xy.c delete XYcoor");
	Tcl_Eval(interp,".c.s.xy.c raise xygrid");
	for(i=0;i<N_XYHITS;i++){
		sprintf(cmd,"DrawXYCell %d %d %s %s {%s rec%d} %d"
			,XYHITS[i].x.id ,XYHITS[i].y.id ,"#FF0000", "#000000" ,"XYcoor", ++recnum,1);
		Tcl_Eval(interp,cmd);
	}

   if(continuous)Tcl_Eval(interp,"update;update;after idle DrawEvent");
  
   return TCL_OK;
}


/************************************/
/* Read an event from shared memory */
/************************************/
int GetEvent(void)
{
	CODA_roc_data_t *roc_data[MAX_ROC_BANKS];
	int Nroc_ptrs;
	unsigned int LRS1881M_slots, LRS1877_slots;
	int err;
	int i,j,k;
	unsigned int adc[64];
	static int event_read_err=0;


	/* read in next event */
	if(strlen(SESSION)){
		if(ReadEventFromET(&EVENT_HEAD)){
			/* problem reading the event. Don't block the program waiting for the next event */
			/* Use the event_read_err flag to write the error message once so we don't */
			/* do it repeatedly if we happen to be in continuous mode. */
			if(!event_read_err)fprintf(stderr,"Event unavailable from ET\n"); 
			event_read_err=1;
			return -1;
		}else{
			event_read_err=0; /* clear the flag once we get a good event */
		}
	}else{
		if(ReadCODAEventFromFile(CODA_FILE,&EVENT_HEAD))return -1;
	}


	NUM_EVENTS_READ++;

	/* Get pointers to the data read from the ROCs */
	Nroc_ptrs = MAX_ROC_BANKS;
	if(GetBankPtrs((char*)EVENT_HEAD, &EVENT_ID, roc_data, &Nroc_ptrs ))return 0;
	NUM_PHYSICS_EVENTS++;

	/* Loop over ROCS, deciphering the 1881M and 1877 data */
	LRS1881M_slots = 0x01<<ADC_SLOT;
	LRS1877_slots  = (0x01<<TDC_SLOT_1) | (0x01<<TDC_SLOT_2);
	for(i=0;i<Nroc_ptrs;i++){

		N_TDC_HITS = MAX_THITS;	
		err = ParseAlldata(roc_data[i],LRS1881M_slots ,adc ,LRS1877_slots ,TDC ,&N_TDC_HITS);
		if(err)continue;

	}

	/* Subtract pedestals */
	for(i=0;i<36;i++)ADC[i] = ((int)adc[i]) - ((int)PEDESTAL[i]);

	/*---- Look for X-Y coincidences ----*/
	/* seperate X and Y hits */
	N_XHITS=N_YHITS=N_XYHITS=0;
	for(i=0;i<N_TDC_HITS;i++){

		/* X-Hits */
		if(TDC[i].slot==TDC_SLOT_1 && TDC[i].id<=63){
			if(N_XHITS<MAX_THITS){
				XHITS[N_XHITS++] = TDC[i];
			}else{
				printf("Too many X-hits. Aborting event.\n");
				break;
			}
		}

		/* Y-hits */
		if(TDC[i].slot==TDC_SLOT_2 && TDC[i].id<=63){
			if(N_YHITS<MAX_THITS){
				YHITS[N_YHITS++] = TDC[i];
			}else{
				printf("Too many Y-hits. Aborting event.\n");
				break;
			}
		}
	}
	/* X-Y-hits */
	for(j=0;j<N_XHITS;j++){
		for(k=0;k<N_YHITS;k++){
			if(abs(XHITS[j].val-YHITS[k].val)<=5){
				if(N_XYHITS<MAX_THITS){
					XYHITS[N_XYHITS].x = XHITS[j];
					XYHITS[N_XYHITS++].y = YHITS[k];
				}else{
					printf("Too many X-Y-hits. Aborting event.\n");
					break;
				}
			}
		}
	}

   return 0;                  /* event read          */
}

/* Read in the configuration files */
int ReadConfiguration(int run_number)
{
	int err;
	int i;
	float fpeds[64];

	err = GetCalibConstants_Float(run_number, "pedestals",36,fpeds);
	for(i=1;i<=36;i++)PEDESTAL[i-1]=(unsigned int)fpeds[i-1];
	
	return err;
}


void ParseCommandLineArgs(int narg,char *argv[])
{
   int i;
   char *argptr;
   
   for(i=1;i<narg;i++){
		argptr = argv[i];
		if(*argptr=='-'){
			argptr++;
			switch(*argptr){
				case 'S':
					strcpy(SESSION,++argptr);
					break;
				case 'h':
				default:
					Usage();
			}
		}else{
			strcpy(INPUT_FILE,argptr);
		}
   }
}


void Usage(void)
{
	fprintf(stderr,"\nUsage:\n\tpev [options] -Ssession | filename\n\n");
	fprintf(stderr,"\toptions:\n\n");
	fprintf(stderr,"\t -Ssession      read events from ET session \"session\"\n");
	fprintf(stderr,"\t -h             print this usage statement\n");
	fprintf(stderr,"\n\n");
	exit(-1);
}







