#!/usr/bin/wish -f

global ARRAYSIZE DRAWSCALE
global continuous use_adc show_dots
global xoffset yoffset xscale yscale
global tag_energy_cut
global reset_histos
global sum_energy_frac
global require_tagger
global ADCSUM_CUT

set continuous 0
set use_adc 1
set show_dots 0

set xoffset 0.0
set yoffset 0.0
set xscale 1.0
set yscale 1.0

set tag_energy_cut 0.0
set reset_histos 0
set sum_energy_frac 30

proc Initialize {asize dscale} {
   global ARRAYSIZE DRAWSCALE
   global continuous use_adc show_dots
   global tag_energy_cut
   global reset_histos
   global sum_energy_frac
   global require_tagger
   global ADCSUM_CUT
   
   set ARRAYSIZE $asize
   set DRAWSCALE $dscale
   
   # top frame and canvas
   frame .c
   set len [expr (${DRAWSCALE}*${ARRAYSIZE})+100]
   canvas .c.c -width $len -height $len -relief sunken -bg white -borderwidth 3
   pack .c.c -side left



   # Bottom buttons
   frame .b
   button .b.quit -text "Quit" -command {destroy .}
   pack .b.quit -side right
   button .b.next -text "Next Event" -command {DrawEvent}
   button .b.cont -text "Continuous" \
      -command {.b.stop configure -state normal ;set continuous 1;DrawEvent}
   button .b.stop -text "Stop" -state disabled \
      -command {.b.stop configure -state disabled ;set continuous 0}
   pack .b.next .b.cont .b.stop -side left
      
   # Cell Energy and time info
   frame .i
   label .i.energy -text "E:" -width 15 -borderwidth 2 -relief groove
   label .i.time -text "T:" -width 15 -borderwidth 2 -relief groove
   label .i.id -text "id:" -width 15 -borderwidth 2 -relief groove
   pack .i.energy .i.time .i.id -side left -fill both

   # TAC table position info
   frame .tt -relief groove -borderwidth 3
   label .tt.x -text "X:" -width 15 -borderwidth 2 -relief groove
   label .tt.y -text "Y:" -width 15 -borderwidth 2 -relief groove
   button .tt.edit -text "Edit offsets and scales" -command {EditOffsets}
   pack .tt.x .tt.y .tt.edit -side left

   # frame beside the canvas
   frame .c.s
   pack .c.s -side right -fill both

   # Whole Detector Info
   set w .c.s.di
   frame $w -borderwidth 3 -relief groove
   label $w.edep -text "Edeposited: 0.000 MeV" -width 25 -anchor w
   label $w.run -text "Run:0" -width 25 -anchor w
   label $w.evnt -text "Event:0" -width 25 -anchor w
   label $w.tageff -text "Tagger eff.:0%" -width 25 -anchor w
   pack $w.edep $w.run $w.evnt $w.tageff -side top -fill both
   pack $w -side top -fill both

   # Individual detector energies
   set w .c.s.erg
   frame $w -relief groove -borderwidth 3
   listbox $w.lb -height 10 -width 25 -yscrollcommand {.c.s.erg.sb set}
   scrollbar $w.sb -command {.c.s.erg.lb yview}
   for {set j 1} {[expr $j<=$ARRAYSIZE]} {incr j} {
      for {set i 1} {[expr $i<=$ARRAYSIZE]} {incr i} {
         $w.lb insert end "E(x=0,y=0): 0.00MeV"
#         set s $w.labx${i}y${j}
#         label $s -text "E(x=0,y=0): 0.00 MeV" -width 25
#         pack $s -side top
      }
   }
   pack $w.sb $w.lb -side left -fill both
   pack $w -side top -fill both

   # Display controls
   set w .c.s.dc
   frame $w -relief groove -borderwidth 3
   checkbutton $w.pd -text "Plot Dots" -variable show_dots -anchor w
   frame $w.dtf
   radiobutton $w.dtf.adc -text "ADC" -variable use_adc -value 1 -anchor w
   radiobutton $w.dtf.energy -text "Energy" -variable use_adc -value 0 -anchor w
   pack $w.dtf.adc $w.dtf.energy -side top -anchor w -fill x
   button $w.rh -text "Reset Histos" -command {set reset_histos 1}
   frame $w.eff
   label $w.eff.lsb -text "% Energy Sum"
   button $w.eff.set -text "set" -width 3 -command {set sum_energy_frac [.c.s.dc.eff.e get]}
   entry $w.eff.e -width 4
   $w.eff.e insert 0 $sum_energy_frac
   bind $w.eff.e <Return> {set sum_energy_frac [.c.s.dc.eff.e get]}
   pack $w.eff.lsb $w.eff.set -side left
   pack $w.eff.e -side right
   frame $w.adcsum
   label $w.adcsum.lsb -text "ADC Sum cut"
   button $w.adcsum.set -text "set" -width 3 -command {set ADCSUM_CUT [.c.s.dc.adcsum.e get]}
   entry $w.adcsum.e -width 7
   $w.adcsum.e insert 0 $ADCSUM_CUT
   bind $w.adcsum.e <Return> {set ADCSUM_CUT [.c.s.dc.adcsum.e get]}
   pack $w.adcsum.lsb $w.adcsum.set -side left
   pack $w.adcsum.e -side right
   checkbutton $w.rt -text "Require Tagger" -variable require_tagger -anchor w
   
   pack $w.pd $w.dtf $w.eff $w.adcsum $w.rt $w.rh -side top -fill both
   pack $w -side top -fill x

   # Analysis controls

	# X-Y coordinate detector
	frame .c.s.xy
	pack .c.s.xy -side top
	canvas .c.s.xy.c -width 192 -height 192 -relief sunken -bg yellow -borderwidth 2
	label .c.s.xy.lab -text "X-Y coordinate detector"
	pack .c.s.xy.c .c.s.xy.lab -side top
	
	pack .c .i .tt .b -side top -padx 5 -pady 5 -fill both

   wm title . "PEV (Primex Event Viewer)"
}

proc DrawGrid {} {
   global ARRAYSIZE DRAWSCALE

   for {set i 0} {[expr $i<$ARRAYSIZE]} {incr i} {
      for {set j 0} {[expr $j<$ARRAYSIZE]} {incr j} {
         DrawCell $i $j #FFF #000 grid 1
      }
   }   

}

proc DrawCell { i j fill outline tags width} {
   global ARRAYSIZE DRAWSCALE

   set scale $DRAWSCALE
   set off   50

   set xoff $off
   set yoff $off
   set xscale $scale
   set yscale $scale
   
   set x [expr $xoff+($xscale*$i)]
   set y [expr $yoff+($yscale*$j)]
   set x2 [expr $x+$xscale]
   set y2 [expr $y+$yscale]

   .c.c create rectangle $x $y $x2 $y2 \
      -fill $fill -outline $outline \
      -tags $tags -width $width
   
}

proc DrawXYCell { i j fill outline tags width} {
   global ARRAYSIZE DRAWSCALE

   set scale [expr $DRAWSCALE/32.0]
   set off   5

   set xoff $off
   set yoff $off
   set xscale $scale
   set yscale $scale
   
   set x [expr $xoff+($xscale*$i)]
   set y [expr $yoff+($yscale*$j)]
   set x2 [expr $x+$xscale]
   set y2 [expr $y+$yscale]

   .c.s.xy.c create rectangle $x $y $x2 $y2 \
      -fill $fill -outline $outline \
      -tags $tags -width $width
   
}

proc DrawXYGrid {} {
	global ARRAYSIZE DRAWSCALE

return

	set scale [expr $DRAWSCALE/32.0]
	set off 5

#   for {set i 0} {[expr $i<64]} {incr i} {
#      for {set j 0} {[expr $j<64]} {incr j} {
#         DrawXYCell $i $j #FFF #000 xygrid 1
#      }
#   }

	set x [expr $off]
	set y [expr $off]
	set x2 [expr $x+64*$scale]
	set y2 [expr $y+64*$scale]

	.c.c create rectangle $x $y $x2 $y2 \
      -fill #AAA -outline #000 \
      -tags xygrid -width 1

}


proc DrawDot { i j fill tags} {
   global ARRAYSIZE DRAWSCALE

   set scale $DRAWSCALE
   set off   50
   set radius 3

   set xoff $off
   set yoff $off
   set xscale $scale
   set yscale $scale

   set x [expr $xoff+($xscale*$i)-$radius]
   set y [expr $yoff+($yscale*$j)-$radius]
   set x2 [expr $x+(2.0*$radius)]
   set y2 [expr $y+(2.0*$radius)]

   .c.c create arc $x $y $x2 $y2 \
      -fill $fill -tags $tags

}

proc EditOffsets {} {

   global xoffset yoffset xscale yscale

   toplevel .ss
   wm title .ss "Edit factors"

   set w .ss.t
   frame $w
   label $w.lab -text "Edit conversion constants for table position"

   frame $w.xo
   label $w.xo.lab -text "X offset:" -width 10
   entry $w.xo.e
   pack $w.xo.lab $w.xo.e -side left -fill both
   frame $w.xs
   label $w.xs.lab -text "X scale:" -width 10
   entry $w.xs.e
   pack $w.xs.lab $w.xs.e -side left -fill both
   pack $w.xo $w.xs -side top -fill both

   frame $w.yo
   label $w.yo.lab -text "Y offset:" -width 10
   entry $w.yo.e
   pack $w.yo.lab $w.yo.e -side left -fill both
   frame $w.ys
   label $w.ys.lab -text "Y scale:" -width 10
   entry $w.ys.e
   pack $w.ys.lab $w.ys.e -side left -fill both
   pack $w.yo $w.ys -side top -fill both
   
   GetOffsets
   $w.xo.e insert 0 $xoffset
   $w.xs.e insert 0 $xscale
   $w.yo.e insert 0 $yoffset
   $w.ys.e insert 0 $yscale
   
   # buttons
   frame .ss.b
   button .ss.b.cancel -text "Cancel" -width 7 -command {destroy .ss}
   button .ss.b.ok -text "OK" -width 7 -command {\
      set xoffset [.ss.t.xo.e get];\
      set xscale [.ss.t.xs.e get];\
      set yoffset [.ss.t.yo.e get];\
      set yscale [.ss.t.ys.e get];\
      SetOffsets;\
      destroy .ss;\
   }
   pack .ss.b.ok -side right
   pack .ss.b.cancel -side left
   pack .ss.t .ss.b -side top -fill both
   
   tkwait window .ss
}


#Initialize
#DrawGrid

#tkwait window .


















