

//===============================================================================
//
// calib_constants.cc
// 
// 8/24/01  D. Lawrence
// 
// The routines in this file provide a means for reading in calibration
// constants for a given run. Initially, they just read them from files
// in a predefined location or one pointed to by the environment. Eventually,
// This should all be in a database.
// 
// 
//===============================================================================


#include <calib_constants.h>

//================================================
// Get the constants referenced by name.
// 
// 
// char *name 		which constants to get
// 
// int N 			number of values to read in
// 
// float *array		where to put the values
// 
// int type 		type of data: 1=float 2=int (use
//                  CALIB_DATA_TYPE_FLOAT or
//                  CALIB_DATA_TYPE_INT)
//================================================
int GetCalibConstants(int runno, char *name,int N, void *array,int type)
{
	int i,j;
	char *ptr, fname[256];
	char path[256]="/data/PARMS";
	FILE *f;
	int err=0;
	char *typestr;
	unsigned int tmp;			// type of this doesn't matter. Address will be typecast
	int *iptr=(int*)&tmp;		// see! we typecast it here ...
	float *fptr=(float*)&tmp;	// ... and here!
	int *iarray=(int*)array;
	float *farray=(float*)array;

	// Override default directory with environment
	if(ptr=getenv("PRIMEX_CONFIG_FILE_PATH"))strcpy(path,ptr);

	// Set type string
	switch(type){
		case CALIB_DATA_TYPE_FLOAT: // float
			typestr = "%f";
			break;
		case CALIB_DATA_TYPE_INT: // int
			typestr = "%d";
			break;
		default:
			cerr<<"Unknown data type specified for \""<<name<<"\" ("<<type<<")"<<endl;
			return -1;
	}

	// Open file to read in constants
	sprintf(fname,"%s/%s.dat",path,name);
	f=fopen(fname,"r");
	if(!f){
		fprintf(stderr,"Unable to open \"%s\"\n",fname);
		return -2;
	}
	printf("Reading %s\n",fname);

	// Loop to read in constants
	for(i=0;i<N;i++){
		fscanf(f,typestr,&tmp);
		switch(type){
			case CALIB_DATA_TYPE_FLOAT: // float
				farray[i]=*fptr;
				break;
			case CALIB_DATA_TYPE_INT: // int
				iarray[i]=*iptr;
				break;
			default:
				cerr<<"Unknown data type specified for \""<<name<<"\" ("<<type<<")"<<endl;
				return -1;
		}
	}
	fclose(f);

	return 0;
}


// Wrappers for above routine
int GetCalibConstants_Float(int runno, char *name,int N, float *array)
{
	return GetCalibConstants(runno, name, N,(void *)array,CALIB_DATA_TYPE_FLOAT);
}

int GetCalibConstants_Int(int runno, char *name,int N, int *array)
{
	return GetCalibConstants(runno, name, N,(void *)array,CALIB_DATA_TYPE_INT);
}



