

//===============================================================================
//
// coda_file.cc
// 
// 8/20/01  D. Lawrence
// 
// The routines in this file can be used to open a CODA formatted data
// file and read in events from it. It only looks at the root bank for each
// event and does not do any further parsing of the data within that bank.
// (Use the routines in coda_parse.cc for that,)
// 
// 
//===============================================================================


#include <coda_file.h>



//===================================================
// Open a CODA formatted file for reading.
// Right now, this is just a wrapper for the
// standard fopen(). There may be cause to 
// add more functionality here in the future,
// so we define it.
//===================================================
FILE* OpenCODAFile(char *fname)
{
	return fopen(fname,"r");
}

//===================================================
// Open a CODA formatted file for reading.
// Right now, this is just a wrapper for the
// standard fclose(). There may be cause to 
// add more functionality here in the future,
// so we define it.
//===================================================
int CloseCODAFile(FILE *file)
{
	return fclose(file);
}


//===================================================
// Read an event from an open file
//===================================================
int ReadCODAEventFromFile(FILE *file,CODA_header_t **evt)
{

#define buff_size 8192  /* maximum event size in bytes */

	static char buff[buff_size];
	CODA_bank_t *evt_ptr;
	int Nattempts;
	int words_read;
	int words_to_read;

	/* Look for valid bank header */
	evt_ptr = (CODA_bank_t*)buff;
	Nattempts = 0;
	do{
		words_read=fread(buff,sizeof(int),2,file);
		if(words_read<2){
			fprintf(stderr,"%s:%d End of file.\n\n",__FILE__,__LINE__);
			return -2;
		}
		if(Nattempts>1000){
			fprintf(stderr,"Failed to find valid header after %d attempts.\n",Nattempts);
			return -3;
		}
	}while((evt_ptr->num != 0xCC) || (evt_ptr->length >32767));

	/* pass back pointer to start of event */
	*evt = (CODA_header_t*)evt_ptr;

	/* Read in rest of event.                        */
	/* Event size doesn't include length word and we */
	/* already read in 1st word included in length   */
	words_to_read = evt_ptr->length - 1;
	if(words_to_read*sizeof(int) + sizeof(CODA_bank_t) >= buff_size-10){
		fprintf(stderr,"Too many words to read (%d)!!\n",words_to_read);
		exit(-1);
	}
	words_read = fread(&buff[sizeof(CODA_bank_t)],sizeof(int),words_to_read,file);
	if(words_read<words_to_read){
		fprintf(stderr,"%s:%d End of file.\n\n",__FILE__,__LINE__);
		return -2;
	}

	return 0;
}



